@@ -1,159 +1,85 @@
 package ExtUtils::CBuilder;
 
-use strict;
-use File::Spec;
-use File::Basename;
-use IO::File;
-use Config;
+use File::Spec ();
+use File::Path ();
+use File::Basename ();
 
-use vars qw($VERSION);
-$VERSION = '0.00_01';
+use vars qw($VERSION @ISA);
+$VERSION = '0.00_02';
 
-sub new {
-  my $class = shift;
-  my $self = bless {@_}, $class;
+# Okay, this is the brute-force method of finding out what kind of
+# platform we're on.  I don't know of a systematic way.  These values
+# came from the latest (bleadperl) perlport.pod.
 
-  while (my ($k,$v) = each %Config) {
-    $self->{config}{$k} = $v unless exists $self->{config}{$k};
+my %OSTYPES = qw(
+		 aix       Unix
+		 bsdos     Unix
+		 dgux      Unix
+		 dynixptx  Unix
+		 freebsd   Unix
+		 linux     Unix
+		 hpux      Unix
+		 irix      Unix
+		 darwin    Unix
+		 machten   Unix
+		 next      Unix
+		 openbsd   Unix
+		 netbsd    Unix
+		 dec_osf   Unix
+		 svr4      Unix
+		 svr5      Unix
+		 sco_sv    Unix
+		 unicos    Unix
+		 unicosmk  Unix
+		 solaris   Unix
+		 sunos     Unix
+		 cygwin    Unix
+		 os2       Unix
+		 
+		 dos       Windows
+		 MSWin32   Windows
+
+		 os390     EBCDIC
+		 os400     EBCDIC
+		 posix-bc  EBCDIC
+		 vmesa     EBCDIC
+
+		 MacOS     MacOS
+		 VMS       VMS
+		 VOS       VOS
+		 riscos    RiscOS
+		 amigaos   Amiga
+		 mpeix     MPEiX
+		);
+
+# We only use this once - don't waste a symbol table entry on it.
+# More importantly, don't make it an inheritable method.
+my $load = sub {
+  my $mod = shift;
+  eval "use $mod";
+  die $@ if $@;
+  @ISA = ($mod);
+};
+
+{
+  my @package = split /::/, __PACKAGE__;
+  
+  if (grep {-e File::Spec->catfile($_, @package, 'Platform', $^O) . '.pm'} @INC) {
+    $load->(__PACKAGE__ . "::Platform::$^O");
+    
+  } elsif (exists $OSTYPES{$^O} and
+	   grep {-e File::Spec->catfile($_, @package, 'Platform', $OSTYPES{$^O}) . '.pm'} @INC) {
+    $load->(__PACKAGE__ . "::Platform::$OSTYPES{$^O}");
+    
+  } else {
+    $load->(__PACKAGE__ . "::Base");
   }
-  return $self;
-}
-
-sub object_file {
-  my ($self, $filename) = @_;
-
-  # File name, minus the suffix
-  (my $file_base = $filename) =~ s/\.[^.]+$//;
-  return "$file_base$self->{config}{obj_ext}";
-}
-
-sub compile_library {
-  my ($self, %args) = @_;
-  die "Missing 'source' argument to compile_library()" unless defined $args{source};
-  
-  my $cf = $self->{config}; # For convenience
-
-  $args{object_file} ||= $self->object_file($args{source});
-  
-  my @include_dirs = map {"-I$_"} (@{$args{include_dirs} || []},
-				   File::Spec->catdir($cf->{installarchlib}, 'CORE'));
-  
-  my @extra_compiler_flags = $self->split_like_shell($args{extra_compiler_flags});
-  my @cccdlflags = $self->split_like_shell($cf->{cccdlflags});
-  my @ccflags = $self->split_like_shell($cf->{ccflags});
-  my @optimize = $self->split_like_shell($cf->{optimize});
-  my @flags = (@include_dirs, @cccdlflags, @extra_compiler_flags, '-c', @ccflags, @optimize);
-  
-  my @cc = $self->split_like_shell($cf->{cc});
-  
-  $self->do_system(@cc, @flags, '-o', $args{object_file}, $args{source})
-    or die "error building $args{object_file} from '$args{source}'";
-
-  return $args{object_file};
-}
-
-sub compile_executable {
-  die "Not implemented yet";
-}
-
-sub have_c_compiler {
-  my ($self) = @_;
-  return $self->{have_compiler} if defined $self->{have_compiler};
-  
-  my $tmpfile = File::Spec->catfile(File::Spec->tmpdir, 'compilet.c');
-  {
-    my $fh = IO::File->new("> $tmpfile") or die "Can't create $tmpfile: $!";
-    print $fh "int boot_compilet() { return 1; }\n";
-  }
-
-  my ($obj_file, @lib_files);
-  eval {
-    $obj_file = $self->compile_library(source => $tmpfile);
-    @lib_files = $self->link_objects(objects => $obj_file, module_name => 'compilet');
-  };
-  warn $@ if $@;
-  my $result = $self->{have_compiler} = $@ ? 0 : 1;
-  
-  foreach (grep defined, $tmpfile, $obj_file, @lib_files) {
-    1 while unlink;
-  }
-  return $result;
-}
-
-sub lib_file {
-  my ($self, $dl_file) = @_;
-  $dl_file =~ s/\.[^.]+$//;
-  $dl_file =~ tr/"//d;
-  return "$dl_file.$self->{config}{dlext}";
-}
-
-sub need_prelink_objects { 0 }
-
-sub prelink_objects {
-  my ($self, %args) = @_;
-  
-  ($args{dl_file} = $args{dl_name}) =~ s/.*::// unless $args{dl_file};
-  
-  require ExtUtils::Mksymlists;
-  ExtUtils::Mksymlists::Mksymlists( # dl. abbrev for dynamic library
-    DL_VARS  => $args{dl_vars}      || [],
-    DL_FUNCS => $args{dl_funcs}     || {},
-    FUNCLIST => $args{dl_func_list} || [],
-    IMPORTS  => $args{dl_imports}   || {},
-    NAME     => $args{dl_name},
-    DLBASE   => $args{dl_base},
-    FILE     => $args{dl_file},
-  );
-  
-  # Mksymlists will create one of these files
-  return grep -e, map "$args{dl_file}.$_", qw(ext def opt);
-}
-
-sub link_objects {
-  my ($self, %args) = @_;
-  my $cf = $self->{config}; # For convenience
-  
-  my $objects = delete $args{objects};
-  $objects = [$objects] unless ref $objects;
-  $args{lib_file} ||= $self->lib_file($objects->[0]);
-  
-  my @temp_files = 
-    $self->prelink_objects(%args,
-			   dl_name => $args{module_name}) if $self->need_prelink_objects;
-  
-  my @linker_flags = $self->split_like_shell($args{extra_linker_flags});
-  my @lddlflags = $self->split_like_shell($cf->{lddlflags});
-  my @shrp = $self->split_like_shell($cf->{shrpenv});
-  my @ld = $self->split_like_shell($cf->{ld});
-  $self->do_system(@shrp, @ld, @lddlflags, '-o', $args{lib_file}, @$objects, @linker_flags)
-    or die "error building $args{lib_file} from @$objects";
-  
-  return wantarray ? ($args{lib_file}, @temp_files) : $args{lib_file};
-}
-
-sub do_system {
-  my ($self, @cmd) = @_;
-  print "@cmd\n";
-  return !system(@cmd);
-}
-
-sub split_like_shell {
-  my ($self, $string) = @_;
-  
-  return () unless defined($string) && length($string);
-  return @$string if UNIVERSAL::isa($string, 'ARRAY');
-  
-  return $self->shell_split($string);
-}
-
-sub shell_split {
-  return split ' ', $_[1];  # XXX This is naive - needs a fix
 }
 
+sub os_type { $OSTYPES{$^O} }
 
 1;
 __END__
-# Below is stub documentation for your module. You better edit it!
 
 =head1 NAME
 
